@@ -1,8 +1,9 @@
 import pandas as pd
-from .mvnmm import MVNMixtureModel
+from mvnmm import MVNMixtureModel
 
-def Run(cov_df, lineages, k_interval=[10,30], n_runs=2, steps=500, lr=0.005,
-        p=0.01, convergence=True, covariance="diag", show_progr=True, random_state=25):
+def run_inference(cov_df, lineages, k_interval=[10,30], n_runs=2, steps=500, lr=0.005,
+        p=0.01, convergence=True, covariance="diag", show_progr=True, 
+        store_grads=True, store_losses=True, random_state=25):
 
     ic_df = pd.DataFrame(columns=["K","run","NLL","BIC","AIC","ICL"])
     losses_df = pd.DataFrame(columns=["K","run","losses"])
@@ -20,9 +21,10 @@ def Run(cov_df, lineages, k_interval=[10,30], n_runs=2, steps=500, lr=0.005,
 
             kk = x_k.params["K"]
 
-            losses_df = pd.concat([losses_df, compute_loss(x_k, kk, run)], ignore_index=True)  # list
+            if store_grads: grads_df = pd.concat([grads_df, compute_grads(x_k, kk, run)], ignore_index=True)
+            if store_losses: losses_df = pd.concat([losses_df, compute_loss(x_k, kk, run)], ignore_index=True)  # list
+            
             ic_df = pd.concat([ic_df, compute_ic(x_k, kk, run)], ignore_index=True)
-            grads_df = pd.concat([grads_df, compute_grads(x_k, kk, run)], ignore_index=True)
 
     return ic_df, losses_df, grads_df
 
@@ -30,10 +32,14 @@ def Run(cov_df, lineages, k_interval=[10,30], n_runs=2, steps=500, lr=0.005,
 def single_run(k, df, lineages, run="", steps=500, covariance="diag", lr=0.001,
     p=0.01, convergence=True, show_progr=True, random_state=25):
 
-    columns = df.columns[df.columns.str.startswith("cov")].to_list()
-    IS = df.IS.to_list()
-
-    x = MVNMixtureModel(k, data=df[columns], lineages=lineages, IS=IS, columns=columns)
+    try:
+        columns = df.columns[df.columns.str.startswith("cov")].to_list()
+        IS = df.IS.to_list()
+        x = MVNMixtureModel(k, data=df[columns], lineages=lineages, IS=IS, columns=columns)
+    except:
+        IS = ["IS.".join(str(i)) for i in range(df.shape[0])]
+        x = MVNMixtureModel(k, data=df, lineages=lineages, IS=IS)
+    
     x.fit(steps=steps, cov_type=covariance, lr=lr, p=p, convergence=convergence, random_state=random_state, show_progr=show_progr)
     x.classifier()
 
